@@ -18,6 +18,7 @@ import sys
 from calculate.core.server.func import Action, Tasks
 from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
 from calculate.lib.cl_template import TemplatesError
+from calculate.lib.datavars import DataVarsError
 from calculate.lib.utils.files import FilesError
 from calculate.lib.variables.system import RootType
 from calculate.update.emerge_parser import EmergeError
@@ -38,7 +39,7 @@ class ClBuilderImageAction(Action):
     """
     # ошибки, которые отображаются без подробностей
     native_error = (DistributiveError, FilesError, UpdateError,
-                    TemplatesError,
+                    TemplatesError, DataVarsError,
                     BuilderError, GitError, EmergeError)
 
     successMessage = __("Image created successfully")
@@ -184,6 +185,27 @@ class ClBuilderImageAction(Action):
               'message': __("Creating package list"),
               'method': 'Builder.create_package_list(cl_builder_path,'
                         'cl_builder_image_filename)',
+              'condition': lambda Get: Get('cl_builder_container_set') == 'off'
+              },
+             {'name': 'container_data',
+              'condition': lambda Get: Get('cl_builder_container_set') == 'on'
+              },
+             {'name': 'container_data:prepare_container_data',
+              'message': __('Preparing container meta data'),
+              'method': 'Builder.prepare_container_data('
+                        'cl_builder_container_data_path)',
+              },
+             {'name': 'prepare_container_data:pack_container_data',
+              'method': 'Builder.pack_container_data('
+                        'cl_builder_container_data_path,cl_builder_image)',
+              },
+             {'name': 'pack_container_data:digest_container_data',
+              'message': __('Calculating SHA256 checksum'),
+              'method': 'Builder.create_digest_container(cl_builder_image)',
+             },
+             {'name': 'prepare_container_data!:clear_container_data',
+              'method': 'Builder.remove_container_data('
+                        'cl_builder_container_data_path)'
               },
              {'name': 'umount_system!:mount_system',
               'warning': _("Restore build system resources"),
@@ -216,6 +238,10 @@ class ClBuilderImageAction(Action):
               'condition': lambda Get: (
                   Get('builder.cl_builder_livemenu_path') != "")
               },
+             {'name': 'pack_container_data:update_meta_index',
+              'message': __('Updating containers index'),
+              'method': 'Builder.update_http_meta(cl_builder_container_path)',
+              },
          ]
          },
         {'name': 'failed',
