@@ -819,6 +819,19 @@ class VariableClBuilderPkgdir(Variable):
         else:
             return ""
 
+class VariableClBuilderPkgdirStable(Variable):
+    """
+    Директория со стабильным набором бинарных пакетов
+    """
+    value_format = "{cl_builder_pkgdir}.stable"
+
+
+class VariableClBuilderPkgdirTrunk(Variable):
+    """
+    Директория со набором бинарных пакетов, осавшихся в результате
+    прерывания сборки
+    """
+    value_format = "{cl_builder_pkgdir}.trunk"
 
 class VariableClBuilderAction(ReadonlyVariable):
     """
@@ -1933,3 +1946,4 @@ class VariableClBuilderContainerDataPath(ImagePathHelper, Variable):
         return self.Get('cl_builder_container_base_path')
 
     prefix = "lxc-"
+
